# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# devoirs.py
#
# Librairie de gestion de la distribution de devoirs
#
###########################################################################


import sys, os, shutil, pickle
sys.path.append('/usr/share/eole/controlevnc')
import ldap_utils

from creole import parsedico
DICO_EOLE = parsedico.parse_dico()
HOME_PATH = DICO_EOLE['home_path']
# chemin vers le répertoire "devoirs" de l'utilisateur
DEV_BASE_DIR = os.path.join(HOME_PATH, '%(user).1s', '%(user)s', 'devoirs')
DEV_PERSO_BASE_DIR = os.path.join(HOME_PATH, '%(user).1s', '%(user)s', 'perso/devoirs')
# le dossier contenant les devoirs distribués par un prof.
DEV_DIST_DIR = os.path.join(HOME_PATH, 'workgroups/devoirs/%s')
# les informations concernant les devoirs (nom, groupe, etc) sont sauvegardés
# dans le fichier /home/<l>/<login>/devoirs/.devoirs.dat
DEV_INFO = os.path.join(DEV_DIST_DIR, 'devoirs.dat')
RESERVED_NAMES = ['.distribues', '.ftp', 'Maildir', 'corrections', 'devoirs', 'devoir', 'donnees', 'groupes', 'perso', 'ramasses']

def load_dict(fich):
    """charge devoirs.dat
    """
    try: return pickle.load(file(fich, 'rb'))
    except: return {}

def save_dict(fich, dico):
    """sauve devoirs.dat
    """
    return pickle.dump(dico, file(fich,'wb'))

def get_devs(username):
    fich = DEV_INFO%username
    return load_dict(fich)

def save_devs(username, devs):
    fich = DEV_INFO%username
    return save_dict(fich, devs)

def copy_dev(rep_path, user_dest_dir, user):
    """copie <rep_path>/* dans /home/<u>/<user>/perso/devoirs/<dev_name>
    crée un lien /home/<u>/<user>/perso/devoirs/<dev_name>/donnees => <rep_path>/donnees
    """
    if os.path.exists(user_dest_dir): shutil.rmtree(user_dest_dir)
    os.makedirs(user_dest_dir)
    for i in os.listdir(rep_path):
        f = os.path.join(rep_path, i)
        if os.path.isfile(f): shutil.copy(f, user_dest_dir)
        # copie les dossiers sauf le dossier "donnees"
        if os.path.isdir(f) and i.strip().lower() != 'donnees':
            shutil.copytree(f, user_dest_dir)
    # crée un lien vers /home/workgroup/<prof>/<devoir>/donnees
    if os.path.isdir(os.path.join(rep_path, 'donnees')):
        os.symlink(os.path.join(rep_path, 'donnees'), os.path.join(user_dest_dir, 'donnees'))
    #dico = { 'user': user, 'rep': user_dest_dir}
    os.system('setfacl -PRm u:%s:rwx %s'%(user, user_dest_dir))
    os.system('setfacl -PRdm u:%s:rwx %s'%(user, user_dest_dir))

################
# DISTRIBUTION #
def distribute(username, groupe, dev_name, eleve_only, in_perso=True):
    """distribue /home/<u>/<username>/devoirs/<dev_name>/ aux membres de <groupe>
    """
    dev_name = dev_name.lower()
    if dev_name in RESERVED_NAMES:
        print 'Nom du devoir invalide %s'%dev_name
        return
    devs = get_devs(username)
    devs[dev_name] = {'groupe': groupe,
                          'eleve_only': eleve_only,
                          'state': 'err'}
    if eleve_only:
        user_liste = ldap_utils.get_membres_croise((groupe, 'eleves'))
    else:
        user_liste = ldap_utils.get_membres(groupe)
    # répertoire contenant le devoir chargé par Gestion-postes
    rep_path = os.path.join(DEV_DIST_DIR%username, dev_name)
    donnees = os.path.join(rep_path, 'donnees')
    # donner au élèves le droit de lire les données
#    if os.path.isdir(donnees): os.system('/usr/bin/setfacl -m g:%s:r-x %s'%(groupe, donnees))
    # distribution des fichiers
    reussi= 0
    rateu = []
    if in_perso:
        dev_dir = DEV_PERSO_BASE_DIR
    else:
        dev_dir = DEV_BASE_DIR
    for user in user_liste:
#        if 1:
        try:
            user_dev_dir = dev_dir%{'user': user}
            user_dev_dir = os.path.join(user_dev_dir, username)
            if not os.path.exists(user_dev_dir): os.makedirs(user_dev_dir)
            user_dest_dir = os.path.join(user_dev_dir, dev_name)
            copy_dev(rep_path, user_dest_dir, user)
            reussi += 1
        except Exception, e:
            rateu.append(user)
    devs[dev_name]['state'] = 'dist'
    devs[dev_name]['in_perso'] = in_perso
    save_devs(username, devs)
    return (reussi, rateu, dev_name)

#############
# RAMASSAGE #
def ramasse(username, dev_name):
    devs = get_devs(username)
    dev = devs[dev_name]
    if dev['eleve_only']:
        user_liste = ldap_utils.get_membres_croise((dev['groupe'], 'eleves'))
    else:
        user_liste = ldap_utils.get_membres(dev['groupe'])
    # répertoire "devoirs" du prof
    dest = os.path.join(DEV_PERSO_BASE_DIR%{'user':username}, 'ramasses', '%s-%s'%(dev_name, dev['groupe']))
    if not os.path.exists(dest): os.makedirs(dest)
    # dans le perso ?
    if devs[dev_name]['in_perso']:
        dev_dir = DEV_PERSO_BASE_DIR
    else:
        dev_dir = DEV_BASE_DIR
    reussi= 0
    rateu = []
    for user in user_liste:
        # dossier /home/<u>/<user>/devoirs
        user_dev_dir = dev_dir%{'user': user}
        # le dossier à ramasser "devoirs/<login_prof>/<nom du devoir>"
        src = os.path.join(user_dev_dir, username, dev_name)
        # où le placer "devoirs\ramasses\<login>"
        dst = os.path.join(dest, user)
        if os.path.exists(dst): shutil.rmtree(dst)
        os.makedirs(dst)
#        if 1:
        try:
            for i in os.listdir(src):
                f = os.path.join(src, i)
                if os.path.isfile(f): shutil.copy(f, dst)
                # os.path.isdir('/un/lien/vers/répertoire') renvoie True ...
                if os.path.isdir(f) and i.lower() != 'donnees':
                    shutil.copytree(f, os.path.join(dst, i))
            reussi += 1
        except Exception, e:
            rateu.append(user)
            print 'Impossible de ramasser le devoir de %s : %s'%(user, e)
    os.system('setfacl -Rm u:%s:rwx %s'%(username, dest))
    os.system('setfacl -Rdm u:%s:rwx %s'%(username, dest))
    devs[dev_name]['state'] = 'ram'
    save_devs(username, devs)
    return (reussi, rateu)

###########
# RENDAGE #
def rendre(username, dev_name):
    """rend le devoir <dev_name> corrigé par <username> aux membres de <groupe>
    """
    # répertoire du devoir corrigé
    try:
        devs = get_devs(username)
        dev = devs[dev_name]
        source = os.path.join(DEV_PERSO_BASE_DIR%{'user':username}, 'ramasses', '%s-%s'%(dev_name, dev['groupe']))
        if devs[dev_name]['in_perso']:
            dev_dir = DEV_PERSO_BASE_DIR
        else:
            dev_dir = DEV_BASE_DIR
        reussi= 0
        rateu = []
        for user in os.listdir(source):
            src = os.path.join(source, user)
            # répertoire "U:\devoirs\<login.prof>\<dev_name>\correction" de l'élève
            user_dev_dir = dev_dir%{'user': user}
            dst = os.path.join(user_dev_dir, username, dev_name, 'correction')
            if os.path.exists(dst): shutil.rmtree(dst)
            p = os.path.split(dst)[0]
            if not os.path.isdir(p): os.makedirs(p)
            try:
    #            for i in os.listdir(src):
    #                shutil.copy(os.path.join(src, i), dst)
                shutil.copytree(src, dst)
                os.system('setfacl -PRm u:%s:rwx %s'%(user, dst))
                os.system('setfacl -PRdm u:%s:rwx %s'%(user, dst))
                reussi += 1
            except Exception, e:
                rateu.append(user)
                print u'Impossible de rendre le devoir a %s : %s'%(user, e)
        devs[dev_name]['state'] = 'ren'
        save_devs(username, devs)
        return (reussi, rateu)
    except Exception, e:
        print '############ %s'%e

###########################
# SUPPRESSION DES DONNEES #
def supprimer(username, dev_name):
    devs = get_devs(username)
    try:
        dev = devs[dev_name]
        # dans le perso ?
        if dev['in_perso']:
            dev_dir = DEV_PERSO_BASE_DIR
        else:
            dev_dir = DEV_BASE_DIR
        if dev['eleve_only']:
            user_liste = ldap_utils.get_membres_croise((dev['groupe'], 'eleves'))
        else:
            user_liste = ldap_utils.get_membres(dev['groupe'])
        for user in user_liste:
            user_dev_dir = DEV_BASE_DIR%{'user': user}
            cible = os.path.join(user_dev_dir, username, dev_name, 'donnees')
            try:
                if os.path.islink(cible): os.remove(cible)
            except:
                continue
    except: pass
    cible = os.path.join(DEV_DIST_DIR%username, dev_name)
    print cible
    if os.path.isdir(cible): shutil.rmtree(cible)
    return True

